@@ -9,7 +9,6 @@
 
 #include "fcitxconfig_export.h"
 
-#include <functional>
 #include <limits>
 #include <string>
 #include <type_traits>
@@ -45,6 +44,13 @@ private:
     std::string externalUri_;
 };
 
+/// An option that launches external tool.
+class FCITXCONFIG_EXPORT SubConfigOption : public ExternalOption {
+public:
+    using ExternalOption::ExternalOption;
+    void dumpDescription(RawConfig &config) const override;
+};
+
 /// Default Constrain with no actual constrain.
 template <typename T>
 struct NoConstrain {
@@ -74,6 +80,21 @@ private:
     std::string tooltip_;
 };
 
+/// For a list of sub config, the field that should be used for display.
+struct ListDisplayOptionAnnotation {
+    ListDisplayOptionAnnotation(std::string option)
+        : option_(std::move(option)) {}
+
+    bool skipDescription() { return false; }
+    bool skipSave() { return false; }
+    void dumpDescription(RawConfig &config) const {
+        config.setValueByPath("ListDisplayOption", option_);
+    }
+
+private:
+    std::string option_;
+};
+
 /**
  * Annotation to be used against String type to indicate this is a Font.
  */
@@ -112,6 +133,14 @@ struct HideInDescription {
     void dumpDescription(RawConfig &) const {}
 };
 
+template <typename Annotation>
+struct HideInDescriptionAnnotation : public Annotation {
+    using Annotation::Annotation;
+    bool skipDescription() { return true; }
+    using Annotation::dumpDescription;
+    using Annotation::skipSave;
+};
+
 /**
  * List Constrain that applies the constrain to all element.
  */
@@ -203,11 +232,12 @@ private:
 /// Default marshaller that write the config RawConfig.
 template <typename T>
 struct DefaultMarshaller {
-    virtual void marshall(RawConfig &config, const T &value) const {
+    DefaultMarshaller() = default;
+
+    void marshall(RawConfig &config, const T &value) const {
         return marshallOption(config, value);
     }
-    virtual bool unmarshall(T &value, const RawConfig &config,
-                            bool partial) const {
+    bool unmarshall(T &value, const RawConfig &config, bool partial) const {
         return unmarshallOption(value, config, partial);
     }
 };
@@ -253,7 +283,7 @@ private:
 template <typename T, typename Constrain = NoConstrain<T>,
           typename Marshaller = DefaultMarshaller<T>,
           typename Annotation = NoAnnotation>
-class Option : public OptionBaseV2 {
+class Option : public OptionBaseV3 {
 public:
     using value_type = T;
     using constrain_type = Constrain;
@@ -262,10 +292,10 @@ public:
            const T &defaultValue = T(), Constrain constrain = Constrain(),
            Marshaller marshaller = Marshaller(),
            Annotation annotation = Annotation())
-        : OptionBaseV2(parent, std::move(path), std::move(description)),
+        : OptionBaseV3(parent, std::move(path), std::move(description)),
           defaultValue_(defaultValue), value_(defaultValue),
-          marshaller_(marshaller), constrain_(constrain),
-          annotation_(annotation) {
+          marshaller_(std::move(marshaller)), constrain_(std::move(constrain)),
+          annotation_(std::move(annotation)) {
         if (!constrain_.check(defaultValue_)) {
             throw std::invalid_argument(
                 "defaultValue doesn't satisfy constrain");
@@ -393,12 +423,37 @@ using KeyListOption = Option<KeyList, ListConstrain<KeyConstrain>,
 /// Shorthand for create a key list constrain.
 static inline ListConstrain<KeyConstrain>
 KeyListConstrain(KeyConstrainFlags flags = KeyConstrainFlags()) {
-    return ListConstrain<KeyConstrain>(KeyConstrain(flags));
+    return {KeyConstrain(flags)};
 }
 
 /// Shorthand for option that will not show in UI.
-template <typename T>
-using HiddenOption = OptionWithAnnotation<T, HideInDescription>;
+template <typename T, typename Constrain = NoConstrain<T>,
+          typename Marshaller = DefaultMarshaller<T>,
+          typename Annotation = NoAnnotation>
+using HiddenOption =
+    Option<T, Constrain, Marshaller, HideInDescriptionAnnotation<Annotation>>;
+
+template <bool hidden, typename T>
+struct ConditionalHiddenHelper;
+
+template <typename T, typename Constrain, typename Marshaller,
+          typename Annotation>
+struct ConditionalHiddenHelper<false,
+                               Option<T, Constrain, Marshaller, Annotation>> {
+    using OptionType = Option<T, Constrain, Marshaller, Annotation>;
+};
+
+template <typename T, typename Constrain, typename Marshaller,
+          typename Annotation>
+struct ConditionalHiddenHelper<true,
+                               Option<T, Constrain, Marshaller, Annotation>> {
+    using OptionType = HiddenOption<T, Constrain, Marshaller, Annotation>;
+};
+
+template <bool hidden, typename T>
+using ConditionalHidden =
+    typename ConditionalHiddenHelper<hidden, T>::OptionType;
+
 } // namespace fcitx
 
 #endif // _FCITX_CONFIG_OPTION_H_
