@@ -9,7 +9,6 @@
 
 #include <memory>
 #include <fcitx-utils/connectableobject.h>
-#include <fcitx-utils/handlertable.h>
 #include <fcitx-utils/macros.h>
 #include <fcitx/event.h>
 #include <fcitx/globalconfig.h>
@@ -21,7 +20,6 @@
 namespace fcitx {
 
 class InputContext;
-class KeyEvent;
 class InstancePrivate;
 class EventLoop;
 class AddonManager;
@@ -33,7 +31,7 @@ class UserInterfaceManager;
 class GlobalConfig;
 class FocusGroup;
 
-typedef std::function<void(Event &event)> EventHandler;
+using EventHandler = std::function<void(Event &event)>;
 
 /**
  * The function mode of virtual keyboard.
@@ -323,6 +321,9 @@ public:
     /// Exit the fcitx event loop
     void exit();
 
+    /// Exit the fcitx event loop with an exit code.
+    void exit(int exitCode);
+
     /// Restart fcitx instance, this should only be used within a regular Fcitx
     /// server, not within embedded mode.
     void restart();
