@@ -1,26 +1,14 @@
-//
-// Copyright (C) 2015~2015 by CSSlayer
-// wengxt@gmail.com
-//
-// This library is free software; you can redistribute it and/or modify
-// it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 2.1 of the
-// License, or (at your option) any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; see the file COPYING. If not,
-// see <http://www.gnu.org/licenses/>.
-//
+/*
+ * SPDX-FileCopyrightText: 2015-2015 CSSlayer <wengxt@gmail.com>
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
+ */
 #ifndef _FCITX_UTILS_EVENT_H_
 #define _FCITX_UTILS_EVENT_H_
 
-#include <time.h>
 #include <cstring>
+#include <ctime>
 #include <functional>
 #include <memory>
 #include <stdexcept>
@@ -38,13 +26,13 @@ enum class IOEventFlag {
     EdgeTrigger = (1 << 4),
 };
 
-typedef Flags<IOEventFlag> IOEventFlags;
+using IOEventFlags = Flags<IOEventFlag>;
 
 class FCITXUTILS_EXPORT EventLoopException : public std::runtime_error {
 public:
     EventLoopException(int error);
 
-    int error() const { return errno_; }
+    FCITX_NODISCARD int error() const { return errno_; }
 
 private:
     int errno_;
@@ -52,33 +40,33 @@ private:
 
 struct FCITXUTILS_EXPORT EventSource {
     virtual ~EventSource();
-    virtual bool isEnabled() const = 0;
+    FCITX_NODISCARD virtual bool isEnabled() const = 0;
     virtual void setEnabled(bool enabled) = 0;
-    virtual bool isOneShot() const = 0;
+    FCITX_NODISCARD virtual bool isOneShot() const = 0;
     virtual void setOneShot() = 0;
 };
 
 struct FCITXUTILS_EXPORT EventSourceIO : public EventSource {
-    virtual int fd() const = 0;
+    FCITX_NODISCARD virtual int fd() const = 0;
     virtual void setFd(int fd) = 0;
-    virtual IOEventFlags events() const = 0;
+    FCITX_NODISCARD virtual IOEventFlags events() const = 0;
     virtual void setEvents(IOEventFlags flags) = 0;
-    virtual IOEventFlags revents() const = 0;
+    FCITX_NODISCARD virtual IOEventFlags revents() const = 0;
 };
 
 struct FCITXUTILS_EXPORT EventSourceTime : public EventSource {
     virtual void setNextInterval(uint64_t time);
-    virtual uint64_t time() const = 0;
+    FCITX_NODISCARD virtual uint64_t time() const = 0;
     virtual void setTime(uint64_t time) = 0;
-    virtual uint64_t accuracy() const = 0;
+    FCITX_NODISCARD virtual uint64_t accuracy() const = 0;
     virtual void setAccuracy(uint64_t accuracy) = 0;
-    virtual clockid_t clock() const = 0;
+    FCITX_NODISCARD virtual clockid_t clock() const = 0;
 };
 
-typedef std::function<bool(EventSourceIO *, int fd, IOEventFlags flags)>
-    IOCallback;
-typedef std::function<bool(EventSourceTime *, uint64_t usec)> TimeCallback;
-typedef std::function<bool(EventSource *)> EventCallback;
+using IOCallback =
+    std::function<bool(EventSourceIO *, int fd, IOEventFlags flags)>;
+using TimeCallback = std::function<bool(EventSourceTime *, uint64_t usec)>;
+using EventCallback = std::function<bool(EventSource *)>;
 
 FCITXUTILS_EXPORT uint64_t now(clockid_t clock);
 
@@ -90,7 +78,7 @@ public:
     bool exec();
     void exit();
 
-    const char *impl();
+    static const char *impl();
     void *nativeHandle();
 
     FCITX_NODISCARD std::unique_ptr<EventSourceIO>
@@ -102,6 +90,8 @@ public:
     addExitEvent(EventCallback callback);
     FCITX_NODISCARD std::unique_ptr<EventSource>
     addDeferEvent(EventCallback callback);
+    FCITX_NODISCARD std::unique_ptr<EventSource>
+    addPostEvent(EventCallback callback);
 
 private:
     const std::unique_ptr<EventLoopPrivate> d_ptr;
